************************************************;
* Change when moving windows-UNIX;
*libname freeze2 '$HOME/DGRP/data.freeze2';  * Unix form;
libname freeze2  "D:\Dropbox\DH\Flywing\DGRP\data.freeze2";  * Windows form ;
************************************************;

* This version reads an example data set GCorrExampleData.sas7bdat;  
* To run, place this file in the directory named freeze2 above.;
/* DATA for this macro
SNP is a unique identifier for the SNP location calculated by adding the FB5 coordinate for the SNP, 
	plus a 10 X 10^9 if the site is on the X chromosome, 21  X 10^9  is on chromosome 2L, etc.  
Chrom		Chromosome arm (not used in the calculations
snparbnum 	a second unique identifier of SNP.  Just the order in the input file.
psnp 		Minor allele frequency
pb			the allele frequency bin the SNP is in. 
nlines 		number of lines with non-missing genotypes
nummiss		number of lines with missing genotype
Ln1-Ln205  	The genotypes of each line, numbered consecutively.  0 = common homozygote, 1 = rare homozygote, .  is missing value.
		All heterozygotes are scored as missing values.;
Other variables give other useful information not essential to this macro.
*/

* Macro datasampsz returns the number of records in a data set.;
%macro datasampsz(dset,returnCode=);
proc contents data=&dset out=nsnps;
data nsnps;
 set nsnps; * Makes a macro variable called nnsps from a data set variable nobs.;
 call symput('dn',trim(left(nobs)));
run;
%let returnCode=&dn;
%mend; *datasampsz;

/* --------------------------------------------- */
/* MACRO GC calculates all correlations within the specified range of minor allele frequencies.
 Parameters are:
cutrsq the cutoff for the minimum r^2 to be saved.
outlibr name of the SAS library for saving results to.
firstp	the number of the first MAF bin whose LD will be calculated. 
	Macro language expects an integer variable, so I divide by 100 to get MAF for the bin. 
lastp  the number of the last MAF bin whose LD will be calculated.
label	a string to add to the output file name
out    an indicator for how severely log output is suppressed. yes=Note creation of all data sets. Anything else =bare minimum log output. 
All Results Viewer output is suppressed. To activate these, remove either of the statements:
 ods html close;	
 ods results off;
or
 ods listing close;

HOWEVER doing so will vastly increase run time, fill hard drives, etc.  
*/
/* ----------------------------------------------  */


%macro gc(cutrsq,outlibr,firstp,lastp,label,out);
 options nomlogic nomprint nosymbolgen;
 %global dn; * variable for carrying data set sample size from datasampsz;
* options mlogic mprint symbolgen; 
 
%let cutr=%sysevalf(%sysfunc(sqrt(&cutrsq)));
%put Square root of &cutrsq = &cutr;
 %let chunk=100;  * Correlations are calculated in chunks of this many SNPs at a time.;
 %let err=0;
 
 %do q = &firstp %to &lastp;  * Major loop over MAF bins. ;
 * First destroy any old output file of the same name as the output from this run.;  
 * Are you SURE you want to do this?;
 %if %sysfunc(exist(&outlibr..gwgcorr&label&q)) %then %do;
   proc datasets lib=&outlibr;
	  delete gwgcorr&label&q;
     quit;
  run;
 %end;
 %let chnk=&chunk;
 * filename logfile "$HOME/dhlustre/DGRP/LD/gcorrsas&label&q..log";  
  filename logfile "D:/Flywing/DGRP/gcorrsas&label&q..log";
 %if &out^=yes %then %do;
  proc printto log=logfile;
run; 
%end;
%let pbin=%sysevalf(&q/100);
%let ptop = %sysevalf(0.01 + &pbin*1.875-1.17*&pbin**2);  * This formula is specific to cutr=0.5, for r^2>0.5.  
		Choice of a different cutoff would be necessary to change cutr.;
 %put pbin= &pbin ptop= &ptop;
* data set fsnps reads in the focal MAF bin.;
  data fsnps;
   set freeze2.GCorrExampleData;
   *set freeze2.f2LDdata;
   where pb=&pbin;
   * if _N_>1000 then stop;  * for debugging, only read in a small number of sites. ;
   rename snparbnum=focalsnp;
   run;

* This macro code will fill up any hard drive with output unless output is suspended: 
 These commands turn off results output to Output window, and Results Viewer.;
 ods listing close; * Turns off Output to Output window.;  
  ods html close;  * These two commands turn off the Results Tracking and Results viewer output.;
 ods results off;
%datasampsz(fsnps);
%let nfsnps=&dn;

* Transpose the focal data before calculating correlations.;
proc transpose data=fsnps prefix=fsnp out=ftsnp; 
   var Ln1-Ln205;
run;

 * data set csnps contains all the sites in higher pbins than the focal pbin, but with MAF< than the maximum value that can yield a correlation
  of r^2>0.5 with those in the focal bin.;
 data csnps;
   set freeze2.GCorrExampleData;
   where pb>&pbin and psnp<=&ptop;
    *  if _N_>1000 then stop; * for debugging, only read in a small number of sites. ;
   rename snparbnum=compsnp;
  run;
 %datasampsz(csnps);
%let ncsnps=&dn; 


***** Now calculate correlations for just the ftsnp data. Output fwgcorr;
 %let nch=%eval((&nfsnps-1)/&chunk+1);  * For the chunk value chosen above, how many chunks will it take to go through the whole bin?;
 %put Focal vs. comparison &pbin &nfsnps &ncsnps %sysfunc(datetime(),datetime18.) ;

 * Loop for all the chunks to run through.;
 %do r=1 %to &nch;
   * Caculate the index  for the first and last SNPs in the current chunk.;
 %let firstf=%eval((&r-1)*&chunk+1); 
 %if &r=&nch %then %do;  * Special case for the last chunk, which may have less than &chunk values.;
		%let lastf=&nfsnps;
		%let chnk=%eval(&lastf-&firstf+1);
	%end;
 %else %let lastf=%eval(&firstf+&chunk-1); 
  proc printto;
  run;
 %put Focal vs. focal chunking: p=&pbin, &nfsnps snps, &r of &nch chunks. This chunk from &firstf to &lastf..  %sysfunc(datetime(),datetime18.);
 %if &out^=yes %then %do;
  proc printto log=logfile;
 run;
 %end;
 ods listing close;
* Calculating the correlations, send to data set fccorr;
ods output PearsonCorr=fccorr;
	proc corr noprob data=ftsnp;
	 var fsnp&firstf-fsnp&lastf;
	 with fsnp&firstf-fsnp&nfsnps;
	run;
ods output close;
 ods listing; 

 * Iml is used to examine the table of correlations in fccorr, pick out those with r^2>0.5.;
proc iml;
 * output has default width of 15;
  reset fw=15;
  * fccorr has the correlations and the sample sizes for each. This reads those in.;
  use fccorr;
	   read all var {
		%do i = &firstf %to &lastf;
		   fsnp&i
		%end;
		} into corr;
		   read all var {
		%do i = &firstf %to &lastf;
		   nfsnp&i
		%end;
		} into corrn;
	  close fccorr; 

   * fsnps has the allele frequency and number of lines scored for each SNP.;
    use fsnps;
     read all var {focalsnp snp nlines psnp} into focalsnp;
    close fsnps;
   * function loc identifies position of elements in the corr matrix that have r>cutoff for recording. ;
   Bigs=loc(corr>&cutr);
   nhigh=ncol(Bigs);
   if nhigh>0 then do;
   	  * row and column calculate the number of the SNPs whose correlations are large.;
	  row=(floor(&firstf -1 +(Bigs-1)/&chnk)+1);
	  column =&firstf -1 +( (Bigs-&chnk*(row- &firstf)) );
	  dw = corr[Bigs];
	  dn = corrn[Bigs];
	  mr = max(row);
	  * For catching some old error.;
	  if mr>nrow(focalsnp) then do;
        print "Focal vs. focal &r of &nch chunks.";
	    print "Focal Row value is too big=" mr nhigh;
		print "In &q, &r of &nch, from &firstf to &lastf..";
	    mbig=max(Bigs);
	    print nhigh mr mbig;
		print Bigs;
		colrow=row//column;
		print colrow;
		call symput('err','1');
	  end;

	* fid1 and fid2 are the actual SNP locations of those involved in high correlations.;
	  fid1 = focalsnp[column,];
	  fid2= focalsnp[row,];
		corres=fid1||fid2||dn||dw;
	 	create fwgcorr from corres[colname={arb1 snp1 snp1n psnp1 arb2 snp2 snp2n psnp2 incommon Dcorr}];
	     append from corres;
		close fwgcorr; 
	end;
	else do;  * No high correlations found in this chunk.;
	  call delete(fgwgcorr);  * Eliminates the output data, which would be replaced if there were high correlations.;
	end;
quit;  * End of IML routine to extract high correlations. ;
%if &err=1 %then %do;
%put error exiting with &firstf &lastf;
	%goto exit;
%end; 
ods listing close;
data fwgcorru;
  set fwgcorr;
  where SNP1<SNP2; * Above retains redundant correlations: SNP1 vs SNP2 and Snp2 vs. SNP1. This prunes to unique ones;
run;
 %if %sysfunc(exist(fwgcorru)) %then %do;
	  %if %sysfunc(exist(&outlibr..gwgcorr&label&q)) %then %do;
	   proc datasets;
		  append base=&outlibr..gwgcorr&label&q data=fwgcorru;
	     quit;
	  run;
	  %end;
	  %else %do;
	  data &outlibr..gwgcorr&label&q (label='DGRP correlations, freeze 2 data. Organized by p similarity.');
	   set fwgcorru;
	  run;
	  %end;
  %end;
%end; *r for focal;
**** Done with the focal by focal calculation.;

*** Now get the data for SNPs with allele frequencies greater than but close to the bin for ftsnp. output gwgcorr;
 %let chnk=&chunk;

  proc printto;
  run;
%put Focal vs. comparison &pbin &nfsnps &ncsnps %sysfunc(datetime(),datetime18.) ;
	 %if &out^=yes %then %do;
	  proc printto log=logfile;
	  run; 
	%end;
* Preparing data.;
proc transpose data=csnps prefix=csnp out=ctsnp; 
   var Ln1-Ln205;
run;
data fc;
 merge ftsnp ctsnp;
 run;
* This code essentially repeats the above, but the correlations are between SNPs in focal MAF bin and the SNPs outside the bin.;
 %let nch=%eval((&nfsnps-1)/&chunk+1);
 %do r=1 %to &nch;
 %let firstf=%eval((&r-1)*&chunk+1);
 %if &r=&nch %then %do;
		%let lastf=&nfsnps;
		%let chnk=%eval(&lastf-&firstf+1);
	%end;
 %else %let lastf=%eval(&firstf+&chunk-1);
  proc printto;
  run;
 %put  Focal vs. comparison chunking:  p=&pbin,  &nfsnps with &ncsnps snps. &r of &nch chunks. Chunk from &firstf to &lastf..  %sysfunc(datetime(),datetime18.);
 %if &out^=yes %then %do;
  proc printto log=logfile;
run; 
 %put  Focal vs. comparison chunking:  p=&pbin,  &nfsnps with &ncsnps snps. &r of &nch chunks. Chunk from &firstf to &lastf..  %sysfunc(datetime(),datetime18.);
%end;
 ods listing close;
* Now calculate correlations between SNPs in focal MAF bin and the SNPs outside the bin.;
	ods output PearsonCorr=fccorrc;
	proc corr noprob data=fc;
	 var fsnp&firstf-fsnp&lastf;
	 with csnp1-csnp&ncsnps;
	run;
	ods output close;
	ods listing;

*  proc printto print='d:\flywing\dgrp\gwcorrdump.lst';
* run;
proc iml;
 * output has default width of 15;
  reset fw=15;
  use fccorrc;
   read all var {
%do i = &firstf %to &lastf;
   fsnp&i
%end;
} into corr;
   read all var {
%do i = &firstf %to &lastf;
   nfsnp&i
%end;
} into corrn; 
   close fccorrc; 

    use fsnps;
     read all var {focalsnp snp nlines psnp} into focalsnp;
   close fsnps;
     use csnps;
     read all var {compsnp snp nlines psnp} into compsnp;
   close csnps;
   Bigs=loc(corr>&cutr);
	nhigh=ncol(Bigs);   
   if nhigh>0 then do;
	   row=floor((Bigs-1)/&chnk)+1;
	   column = &firstf -1 + ( (Bigs-&chnk*(row-1)) );
	   dw = corr[Bigs];
	  dn = corrn[Bigs];
	  mr = max(row);
	  if mr>nrow(compsnp) then do;
		print "focal vs. comparison in &q, &r of &nch, from &firstf to &lastf..";
	    mbig=max(Bigs);
	    print nhigh mr mbig;
		print Bigs;
		colrow=row//col;
		print colrow;
		call symput('err','1');
	  end;
	  fid = focalsnp[column,];
	  cid= compsnp[row,];
		corres=fid||cid||dn||dw;
	 	create gwgcorr from corres[colname={arb1 snp1 snp1n psnp1 arb2 snp2 snp2n psnp2 incommon dcorr}];
	     append from corres;
		close gwgcorr; 
	end;
	else do;
	  * No high correlations are detected in this chunk, so delete the old gwgcorr file, so it doesnt get 
			appended twice.;
	  %if %sysfunc(exist(gwgcorr)) %then %do; 
		call delete(gwgcorr);
	  %end;
	  print 'No high correlations.';
	end;
**** done with the focal vs. comparison calculations.;

quit;
%if &err=1 %then %do;
%put error exiting with &firstf &lastf;
	%goto exit;
%end; 
 ods listing close;
 %if %sysfunc(exist(gwgcorr)) %then %do;
 	%datasampsz(gwgcorr);
	%let nco=&dn;
	  %if %sysfunc(exist(&outlibr..gwgcorr&label&q)) %then %do;
		   proc datasets;
			  append base=&outlibr..gwgcorr&label&q data=gwgcorr;
			quit;
			  run;
	  %end;
	  %else %do;
		  data &outlibr..gwgcorr&label&q (label='DGRP correlations, freeze 2 data. Organized by p similarity.');
		   set gwgcorr;
		  run;
	  %end;
  %end;
  proc printto;
  run;
  %put &nco %sysfunc(datetime(),datetime18.);
	 %if &out^=yes %then %do;
		  proc printto log=logfile;
		run; 
		%end;
  %end; *r; 
 %if &out^=yes %then %do;
  dm'odsresults; clear';
  proc printto;
  run;
%end; 
%end; * q;
 %exit: ods listing;
  ods html;
  ods results on;
  proc printto;
  run;
%mend;

* indices for starting and ending p bins, label.;
%*macro gc(cutr,outlibr,firstp,lastp,label,out);

%gc(0.5,work,10,11,f22052,no); 
 
