/* Calculating limits on frequencies that can be correlated at r^2>0.5; */
* MAC is minor allele count.;
* In these calculations we assume that the number of genotypes is 205, so minor allele counts can go from 1 to 103.;
* We are only interested in homozygous or haploid genotypes, so no heterozygotes.;

proc iml;
* This generates genotypic identities that have the maximum possible correlations.;
 vecs = j(205,103,0);
do i = 1 to 103;
   do j=1 to i;
	  vecs[j,i]=1;
	end;
 end; 
	 	create fc from vecs;
	     append from vecs;
		close fc;
quit;
ods output PearsonCorr=minmaxcorr;
proc corr noprob data=fc;
 var col1-col103;
run;
ods output close;
* Data set minmaxcorr has the correlations between maximally correlated SNPs at each MAC.;

* How do these counts line up with the binning allele frequency to the nearest 0.01? We call these p bins.;
proc iml;
  ppbin=j(103,3,0);
  do i=1 to 103;
    ppbin[i,1]=i;
    ppbin[i,2]=i/205;
	* pbin value;
    ppbin[i,3] = round(ppbin[i,2],0.01);
   end;
   print ppbin;
   create pbins from ppbin[colname={mac maf pbin}];
  append from ppbin;
quit;
proc means min max noprint;
 by pbin;
 var mac;
 output out=pbinlim min=lowcount max=highcount;
run;
* Data set pbinlim has the highest and lowest MAC that lands a site in a particular p bin.;
 
 proc iml;
  use minmaxcorr;
   read all into corrs;
  close minmaxcorr;
  * Use R^2 as the criterion.;
 corrs=corrs#corrs;
 use pbinlim;
  read all into pbinlim;
  close pbinlim;
  limits=j(51,10,0);
  do k=1 to nrow(limits);
  * Low limit calculated from low MAC for that bin.;
   i =  pbinlim[k,4];
	do j=i to 1 by -1;
	  if corrs[i,j]>0.5 then limits[k,1]=(j)/205;
    end;
  * High limit clculated from high MAC for that bin.;
   i =  pbinlim[k,5];
	do j=i to 103;
	  if corrs[i,j]>0.5 then limits[k,2]=(j)/205;
    end; 
	  limits[k,3]=limits[k,1]/pbinlim[k,1];
	  limits[k,4]=limits[k,2]/pbinlim[k,1];
	  * limits for 205 lines.;
	  limits[k,5]=0.004+pbinlim[k,1]*0.44+pbinlim[k,1]**2*0.41;
	  limits[k,6]=0.01+pbinlim[k,1]*1.875+pbinlim[k,1]**2*-1.17;
	  limits[k,7]=pbinlim[k,1]**2;
	  * Old equation used at the beginning.;
	  limits[k,8]=pbinlim[k,1]/(0.595 + pbinlim[k,1]*0.3);
	  *Checks that predicted low is really < than actual low.;
	  limits[k,9]=limits[k,1]-limits[k,5];
	  *Checks that predicted high is really > than actual high.;
	  limits[k,10]=limits[k,6]-limits[k,2];
  end;
  pbinlimit=pbinlim[,1] || pbinlim[,4:5] || limits;
  print pbinlimit[format=6.3];
  create pbinlimits from pbinlimit[colname={pbin lowcount highcount lowl upl ratiolow ratioup predlowl predupl pbinsq oldpred sftlow sfthigh pbin}];
  append from pbinlimit; 
 quit;
 * Graphs to help diagnose whether limits are correct.;
 proc plot data=pbinlimits;
  plot lowl*pbin predlowl*pbin='-' /overlay;
  plot upl*pbin predupl*pbin='-' oldpred*pbin='o' / overlay;
  plot sftlow*pbin='L' sfthigh*pbin='H' / overlay;
 run;
 proc reg data=pbinlimits(where=(pbin>0.03));
  model lowl=pbin pbinsq;
 proc reg data=pbinlimits(where=(pbin<0.35));
  model upl=pbin pbinsq;
  run; 
